#!/usr/bin/perl
#
# $Id: cf.pm,v 1.12 2006/02/06 16:12:35 rodney Exp $
#

package acctools::cf;

use acctools::tset;
use strict;


sub new {
    my ($this,$c) = @_;
    my $class = ref($this) || $this;
    my $self = {};
    bless($self, $class);

    if (ref($c) eq "HASH" && $c->{cf})
    {
	$self->{tset} = new acctools::tset($c->{cf});
    }
    elsif (my $cf_file = $ENV{'ACCTOOLS_CF'})
    {
	if (! -f $cf_file)
	{
	    die("No such file '$cf_file' given in ACCTOOLS_CF\n");
	}
	$self->{tset} = new acctools::tset($cf_file);
    }
    elsif (-f (my $cf_file = "/usr/local/lib/accounting/cf"))
    {
	$self->{tset} = new acctools::tset($cf_file);
    }
    else
    {
	$self->{tset} = new acctools::tset("/prj/projects/accounting/lib/staff-account");
    }

    



    if (ref($c) eq "HASH")
    {
	$self->{debug} = $c->{debug};
    }

    return($self);
} # sub new()




sub get
{
    my ($self,$tag) = @_;



    my $result = $self->{tset}->get($tag);

    # Fetch from DB, unless known locally.
    if (!defined($result) && $self->{dbh})
    {
	my $accschema = $self->{tset}->get("AccSchema");
	($result) = $self->{dbh}->selectrow_array("SELECT val FROM $accschema.cf WHERE tag = ?",undef, $tag);
    }


    return($result);
}


sub connect
{
    my ($self) = @_;

    my $cf = $self;

    my $uname = `uname -n`; chomp($uname);
    
#    my $USER = $cf->get("AccSchema") || die("could not get AccSchema cf entry");
    
    my $cstring = $cf->get("Connect-String");
    $cstring = $cf->get("Connect-String5.6") if ($] =~ /^5\.006/);
    $cstring = $cf->get("Connect-String5.8") if ($] =~ /^5\.008/);
    
    if (my $cs = $cf->get("Connect-String-$uname"))
    {
	$cstring = $cs;
    }

    $cstring = $cf->get("Cstring") unless ($cstring);


    # FIXME: Streamline our config files!
    my $user = $cf->get("User");
    $user = $cf->get("OraUser") unless ($user);

    my $pass = $cf->get("Password");
    $pass = $cf->get("OraPw") unless ($pass);


    if ($self->{debug})
    {
	print STDERR "logging in to '$cstring', user '$user'\n";
    }

    
    my $dbh = DBI->connect($cstring,
			   $user,
			   $pass,
                           { RaiseError => 1,
			     AutoCommit => 0,
                             LongReadLen => 128000});

    $self->{dbh} = $dbh;

    die("DB logon failed $DBI::errstr") if (!$dbh);
    
    return($dbh);
}



1;
