#!/usr/bin/perl
#
# $Id: cprice.pm,v 1.15 2008/07/31 14:09:00 rodney Exp $
#
# Zweck:
# o Auftrennen von zeitorientiert abgerechneten
#   Verbindungen in verschiedene Tarifklassen
#

package acctools::cprice;

use strict;

#BEGIN {
#    use Exporter   ();
#    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, @EXPORT_FAIL);
#
#               # set the version for version checking
#    $VERSION     = 1.00;
#               # if using RCS/CVS, this may be preferred
#    $VERSION = do { my @r = (q$Revision: 1.15 $ =~ /\d+/g); sprintf "
#%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker
#
#    @ISA         = qw(Exporter);
##    @EXPORT      = qw(&func1 &func2 &func4);
#    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
#
#               # your exported package globals go here,
#               # as well as any optionally exported functions
##    @EXPORT_OK   = qw(cpriceFactory);
##    @EXPORT_FAIL = grep { ! defined &$_ } @EXPORT_OK;
#}
#
#our @EXPORT_OK;






# Constructor
sub new {
    my ($this) = shift;
    my $class = ref($this) || $this;
    my $self = {};
    bless $self, $class;
#    $self->initialize();


    $self->{cf} = shift || die("cannot shift");
    die("missing configuration") unless (ref($self->{cf}) eq "HASH");

    $self->init;

    return $self;
} # sub new()




sub init
{
    my ($self) = @_;

    # Abkuerzung
    $self->{debug} = $self->{cf}->{debug};

#    # Checken, ob die Zeitzonen korrekt angegeben sind
#    $self->{tborders} = []; # Ref on empty array

#    # Borders are valid from ... to ... ?
#    $self->{tvalid} = []; # Ref on empty array


    # Array for Special-Zones ("Sondertage")
    $self->{specialzone} = [] unless defined($self->{specialzone});


#    # Dump zone content ---------------
#    my $zone_i = 0; # Zone-Index
#    foreach my $z (@{$self->{cf}->{timezones}})
#    {
#	if ($self->{debug})
#	{
#	    print "Zonetable $zone_i: --------------\n";
#	    print "Duration: " . $z->getDuration . " sec\n";
#	}
#
##	$self->{tborders}->[$zone_i] = {};
##	$self->{tvalid}->[$zone_i] = {};
#
#	$zone_i++;
#    }

    print "\n\n" if ($self->{debug});
}






sub analyze
{
    my ($self,$c) = @_;

    if ($self->{debug})
    {
	print "analyze: [" . localtime($c->{start}) . '     - ' . localtime($c->{end}) . "    ]\n";
	print "         [" . gmtime($c->{start}) . ' GMT - ' . gmtime($c->{end}) . " GMT]\n";
    }

    my $ret = $self->_analyze($c);

    print $self->zdump($ret) if ($self->{debug});

    return($ret);
}










sub _analyze
{
    my ($self,$c) = @_;

    my @zname; # Current zone names

    my $ret = [];

    # Alle Zonen-Pointer auf den Start-Zeitpunkt der Verbindung
    # (uebergeben in $c->{start}) setzen:
    foreach my $z (@{$self->{cf}->{timezones}})
    {
	$z->jump($c->{start}); # Setze Zonen-Pointer
	push(@zname, $z->getZone()->{name});
    }


    # Letzten Zonennamen merken, damit vermieden werden kann, zwei
    # Zonen gleichen Namens hintereinander zurueckzuliefern.
    my $lastZname = join("-",@zname);

#    print "zone start: $lastZname\n" if ($self->{cf}->{debug});

    # Namen der Start-Zone auf @$ret schieben.
    push(@$ret,{ znames => \@zname,
		 start => $c->{start} });



    for (;;)
    {
	# o Naechstliegende Zonengrenze finden.
	# o Dazu: alle Zonen (dtzone, wtzone usw.) durchgehen und den
	#   getNext()-Wert finden, der am naechsten in der Zukunft liegt.
	my $nextborder;
	foreach my $z (@{$self->{cf}->{timezones}})
	{
	    my $next = $z->getNext;
	    # Result: $next = { start => 123456789, name => 'zoneName' }

	    #
	    # Liegt die in $next->{start} erhaltene Zonengrenze naeher als
	    # eine bereits in $nextborder geschriebene?
	    #
	    if ($next->{start} < $nextborder || !$nextborder)
	    {
		$nextborder = $next->{start};

		if ($self->{cf}->{debug} > 3)
		{
#		    print localtime($next->{start}) . " -> ";
		    print gmtime($next->{start}) . " GMT " . $z->getName .  "-> '$next->{name}'\n";
		}
	    }
	}

	last if ($nextborder > $c->{end});

	my @zname; # Das ist noetig, wg. ueber @$ret zurueckgegebene Ref.

#	last; # DEBUG


	#
	# In $nextborder steht jetzt der Zeitpunkt der naechstliegenden
	# Zonengrenze.
	#
	# Fuer jede Zone $z, fuer die $nextborder einen neuen Zonenstart
	# markiert, wird auf diese Zone vorgesprungen.
	#
	my $cnt;
	foreach my $z (@{$self->{cf}->{timezones}})
	{
	    #
	    # Weiterspringen?
	    #
	    if ($nextborder == $z->getNext->{start})
	    {
#		print "jumpnext for " . $z->getName() . "\n"; # DEBUG
		$z->jumpNext;
	    }
	    else
	    {
#		print "no jump yet for " . $z->getName . "\n"; # DEBUG
	    }

	    push(@zname, $z->getZone->{name});
	}

	# Falls die Folgezone gleich der bestehenden Zone
	# ist, kein Push auf den Stack (redundante Information).
	if ($lastZname ne join("-",@zname))
	{
	    $lastZname = join("-",@zname); # Merken, welches die letzte gepush()te Zone war
	    push(@$ret,{ znames => \@zname,
			 start => $nextborder });
	    
#	    print gmtime($nextborder) . " GMT $lastZname\n";
	}
	
    } # for (;;)

    # Ende-Datum und Duration fuer die jeweilige Zone setzen
    for (my $n = 0; $n < $#$ret; $n++)
    {
	$ret->[$n]->{end} = $ret->[$n + 1]->{start};
	$ret->[$n]->{duration} = $ret->[$n]->{end} - $ret->[$n]->{start};
    }
    $ret->[$#$ret]->{end} = $c->{end};
    $ret->[$#$ret]->{duration} = $c->{end} - $ret->[$#$ret]->{start};


    return($ret);
}



sub zdump
{
    my ($self,$zones) = @_;

    my %dur;

    my $msum0;

#    print "zdump zone list:\n";

    my $ret;

    my $zcnt = 0;
    foreach my $z (@$zones)
    {
	$zcnt++;
	$ret .= gmtime($z->{start}) . " GMT [$z->{start}] -> ";
	$ret .= gmtime($z->{start} + $z->{duration}) . " [" . ($z->{start} + $z->{duration}) . "] ";

	my $zname = join("-",@{$z->{znames}});
	$ret .= "$zname ($z->{duration} s)\n";

	$dur{$zname} += $z->{duration};
	$msum0 += $z->{duration};
    }


    my $msum;
    foreach my $k (keys %dur)
    {
	$ret .= "$k $dur{$k}\n";
	$msum += $dur{$k};
    }

    $ret .= "msum: $msum, zcnt: $zcnt\n";


    if ($msum0 != $msum)
    {
	$ret .= " msum0 $msum0 != msum $msum !!!!!!!!!!!!\n";
    }


    return($ret);
}



1;






