#!/usr/bin/perl
#
# $Id: szmodel.pm,v 1.3 2004/07/07 12:17:45 rodney Exp $
#
# Modell fuer eine Zeitzone, Listen-basiert
# o Modelliert wird eine Zone z.B. fuer Sondertage.
#
# Interface "nach draussen" sind die Methoden:
#
# $z->jump($zeitpunkt)
# $z->getNext()
# $z->jumpNext()
# ...
#

package acctools::cprice::szmodel;

use Carp;
use strict;

# Inherit from cprice::zone
use acctools::cprice::zone;
our @ISA = qw(acctools::cprice::zone);


# Constructor
sub _init {
    my ($self) = @_;

#    $self->{cf} = shift || die("cannot shift");

#    $self->{debug} = $self->{cf}->{debug};

#    die("missing configuration") unless (ref($self->{cf}) eq "HASH");

    $self->{point} = 0; # Zeitpunkt


    # Wie heisst die Zone, wenn gerade kein Sondertag aktiv ist?
    $self->{normal} = "normal";


    # Zonen-DB aufbauen -------------------------------------------


    my $maxpoint = 0;
    my @zlist;

    # Start ist die "normal" zone
    push(@zlist,{ start => 0,
		  name  => $self->{normal} });

    foreach my $z (sort { $a->{start} <=> $b->{start} } @{$self->getCf->{zones}})
    {
	push(@zlist,{ start => $z->{start},
		      name  => $z->{name} });

	my $zone_end = $z->{start} + $z->{duration};  # Endzeitpunkt d. Zone
	confess("bad start/duration") unless ($zone_end > $z->{start});

	push(@zlist,{ start => $zone_end,
		      name  => $self->{normal} });

	if ($zone_end < $maxpoint)
	{
	    die("overlapping zones not allowed.");
	    $maxpoint = $zone_end;
	}
    }


    push(@zlist,{ start => 1950991200, name => $self->{normal} }); # FIXME


    $self->{zlist} = \@zlist;


    if (1)
    {
	print "START szmodel zone dump -------------------------\n";
	foreach my $z (@zlist)
	{
	    print "{ start => $z->{start}, name => '$z->{name}' }\n" if (1); # DEBUG
	}
	print "END szmodel zone dump ----------------------------\n";
    }
	



    $self->{zlist_point} = 0;

    return($self);
} # sub new()



# Hash liefern, das die aktuelle Zone enthaelt
#
#
#   my $h = $self->getZone(788943600);   # liefert die Zone zu einem
#                                        # bestimmten Zeitpunkt
#
#   my $h = $self->getZone();            # liefert die Zone zum aktuellen, d.h.
#                                        # dem in $self->{point} enthaltenen
#                                        # Zeitpunkt.
#
#   Inhalt von $h:
#
#   $h = {
#           start => 788943600,
#           zone  => 'SaSo'
#        };
#
#
sub getZone
{
    my ($self,$t) = @_;

    die ("getZone(t) not supported") if ($t);

    my @zlist = @{$self->{zlist}};

#    print "zlist-point $self->{zlist_point}\n";

    return($zlist[$self->{zlist_point}]);
} # getZone()




# Auf Zeitpunkt "huepfen"
sub jump
{
    my ($self,$t) = @_;

    my @zlist = @{$self->{zlist}};
    $self->{zlist_point} = 0;
    for (my $n = 0; $n < $#zlist; $n++)
    {
	last if ($zlist[$n]->{start} > $t);

	$self->{zlist_point} = $n;
#	print "n = $n\n"; # DEBUG
    }

#    print "szmodel->jump($t) -> " . $self->{zlist_point} . "\n";
}




#
# Naechste Zonengrenze als Hash-Array liefern
#
sub getNext
{
    my ($self) = @_;

    my @zlist = @{$self->{zlist}};

    # Ende der Liste erreicht?
#    print "zlist count: $#zlist\n";
    confess("Out of time") if ($self->{zlist_point} + 1 > $#zlist);

    my $ret = $zlist[$self->{zlist_point} + 1];

#    print "szmodel: nextborder " . gmtime($ret->{start}) . " GMT, '$ret->{name}'\n";

    return($ret);
}



sub XXXjumpNext
{
    my ($self) = @_;

#    print "szmodel: jumpNext\n"; # DEBUG

    my $n = $self->getNext;

    $self->jump($n->{start});
}





1;
