#!/usr/bin/perl
#
# $Id: tzmodel.pm,v 1.7 2004/08/03 15:17:42 rodney Exp $
#
# Modell fuer eine Zeitzone
# o Modelliert wird eine zyklische Zone, z.B. Tage, Wochen.
# o Zonen haben derzeit (Stand 06.07.04) eine feste
#   Laenge, d.h. z.B. Monate funktionieren nicht.
#
#

package acctools::cprice::tzmodel;

use strict;
use Carp;


# Inherit from cprice::zone
use acctools::cprice::zone;
our @ISA = qw(acctools::cprice::zone);


# Nothing to init?
sub _init
{
}



# Hash liefern, das die aktuelle Zone enthaelt: (faktisch identisch mit $self->getNext)
#
#
#   my $h = $self->getZone(788943600);   # liefert die Zone zu einem bestimmten Zeitpunkt
#
#   my $h = $self->getZone();            # liefert die Zone zum aktuellen, d.h.
#                                        # dem in $self->{point} enthaltenen Zeitpunkt.
#
#   Inhalt von $h:
#
#   $h = {
#           start => 788943600,
#           zone  => 'SaSo'
#        };
#
#
sub getZone
{
    my ($self,$t) = @_;

    my $zones = $self->{cf}->{zones};

    $t = $self->{point} unless ($t);

    my $ret = {};
    
    my $dstoff = 0;
    {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
	    localtime($t);
	if ($isdst)
	{
	    $dstoff = 3600; # FIXME!!!
#	    print localtime($t) . " -> isdst\n";
	}
    }

    # Wieviele Sekunden rechts vom Start der aktuellen Zone?
    my $t0 = ($t - ($self->{cf}->{origin} - $dstoff))
	% $self->{cf}->{duration};


    # Start d. Zone finden (z.B. Montag 00:00:00 Uhr fuer Week-Zone)
    my $t_s = $t - $t0;
    
    foreach my $z (reverse(@$zones))
    {
	if ($self->{cf}->{debug} > 2)
	{
	    print "getZone: zstart $z->{start} <= t0 $t0\n";
	}

        if ($z->{start} <= $t0)
        {
            $ret->{start} = $z->{start};
            $ret->{name} = $z->{name};
	    last;
        }
    }

    # Falls keine Zonengrenze gefunden wurde: Wraparound.
    if (!defined($ret->{start}))
    {
#        print "getZone: nomatch\n"; # DEBUG
        $ret->{start} = $zones->[$#$zones]->{start} - $self->{cf}->{duration};
        $ret->{name} = $zones->[$#$zones]->{name};
    }

    if ($self->{cf}->{debug}) # FIXME
    {
	print "getZone($self->{cf}->{name}," . localtime($t) . "): $ret->{name}\n"; # FIXME
    }

    $ret->{start} += $t_s;

    return($ret);
} # getZone()


sub getDuration
{
    my ($self) = @_;
    return($self->{cf}->{duration});
}


# Naechste Zonengrenze liefern
sub getNext
{
    my ($self,$t) = @_;

    confess("$t not supported in getNext") if ($t);

    my $zones = $self->{cf}->{zones};

    $t = $self->{point};

    my $dstoff = 0;
    {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
	    localtime($t);
	if ($isdst)
	{
	    $dstoff = 3600; # FIXME!!!
#	    print localtime($t) . " -> isdst\n";
	}
    }

    
    # Wieviele Sekunden rechts vom Start der aktuellen Zone?
    my $t0 = ($t - ($self->{cf}->{origin} - $dstoff)) % $self->{cf}->{duration};

    # Start d. Zone finden (z.B. Montag 00:00:00 Uhr fuer Week-Zone)
    my $t_s = $t - $t0;

    my $ret = {};


    foreach my $z (@$zones)
    {
	# Ist der Start dieser Zone $z bereits groesser als $t0?
	# Wenn ja, dann ist die Folgezone die naechste.
	if ($z->{start} > $t0)
	{
	    $ret->{start} = $z->{start};
	    $ret->{name} = $z->{name};
	    last;
	}
    }


    # Falls keine Zonengrenze gefunden wurde: Wraparound.
    if (!defined($ret->{start}))
    {
#	print "nomatch\n";
	$ret->{start} = $zones->[0]->{start} + $self->{cf}->{duration};
	$ret->{name} = $zones->[0]->{name};
    }

    $ret->{start} += $t_s;

    if ($self->{cf}->{debug} > 2)
    {
	print "getNext($self->{cf}->{name},";
	print localtime($t) . ") = " . localtime($ret->{start}) . " - $ret->{name}\n";
    }

    return($ret);
}



# Auf Zeitpunkt "huepfen"
sub jump
{
    my ($self,$t) = @_;

    $self->{point} = $t;
}


1;






