#!/usr/bin/perl
#
# $Id: zone.pm,v 1.1 2008/07/31 14:10:39 rodney Exp $
#
# Modell fuer eine Zeitzone (Template)
#
# Interface "nach draussen" sind die Methoden:
#
# $z->jump($zeitpunkt)
# $z->getNext()
# $z->jumpNext()
# ...
#

package acctools::cprice::zone;

use Carp;
use strict;

# Constructor
sub new {
    my ($this) = shift;
    my $class = ref($this) || $this;
    my $self = {};
    bless $self, $class;

    $self->{cf} = shift || die("cannot shift");

    die("missing configuration") unless (ref($self->{cf}) eq "HASH");

    $self->_init;

    return($self);
} # sub new()


sub _init
{
    confess("call to abstract method _init");
}


sub getCf
{
    my ($self) = @_;
    return($self->{cf});
}


# Return zone name
sub getName()
{
    my ($self) = @_;
    return($self->getCf->{name});
}




# Hash liefern, das die aktuelle Zone enthaelt
#
#
#   my $h = $self->getZone(788943600);   # liefert die Zone zu einem
#                                        # bestimmten Zeitpunkt
#
#   my $h = $self->getZone();            # liefert die Zone zum aktuellen, d.h.
#                                        # dem in $self->{point} enthaltenen
#                                        # Zeitpunkt.
#
#   Inhalt von $h:
#
#   $h = {
#           start => 788943600,
#           zone  => 'SaSo'
#        };
#
#
sub getZone
{
    my ($self,$t) = @_;
    die("must be overloaded");
} # getZone()




# Auf Zeitpunkt "huepfen"
sub jump
{
    my ($self,$t) = @_;

    die("must be overloaded");
}




#
# Naechste Zonengrenze als Hash-Array liefern
#
sub getNext
{
    my ($self) = @_;
    die("must be overloaded");
}


# Sprung zur naechsten Zone
sub jumpNext
{
    my ($self) = @_;

    my $z = $self->getNext;

    if ($self->getCf->{debug} > 2)
    {
	print "jumpNext($self->{cf}->{name}) -> " . localtime($z->{start}) . "\n";
    }

    $self->jump($z->{start});
}


# Return zone name
sub getName()
{
    my ($self) = @_;
    return($self->getCf->{name});
}



1;







