#
# $Id: accounter.pm,v 1.7 2004/06/02 17:11:56 rodney Exp $
#
#
# Zweck: Abbildung der Accmod-Funktionalitaeten
#
#

package acctools::flexacc::accounter;

use strict;
use Safe;
use acctools::rgpos;     # Interface to accounting tools.


sub new {
    my ($this,$h) = @_;
    my $class = ref ($this) || $this; # class or object
    my $self = {};

    bless($self, $class);

    $self->{AccmodLibDir} = $h->{AccmodLibDir};

    #
    # OrderParFunc ist eine Funktion, die bei einem
    # uebergebenen Auftragsnamen ein Hash der Form
    #
    # my %h = { name => 'wert1',
    #           vorname => 'wert2' }
    #
    # zurueckliefert.
    #
    # Bei LF.net stehen die OrderParameters in der Table AUFTRAG_PAR.
    #
    $self->{OrderParFunc} = $h->{OrderParFunc};
    if (ref($self->{OrderParFunc}) ne "CODE")
    {
	die("OrderParFunc must be a code reference");
    }

    # FIXME. Should only unshift if AccmodLibDir is not
    # already contained in @INC.
    unshift(@INC,$self->{AccmodLibDir});

    return($self);
}


sub ping
{
    print "self-ping\n";
}



# Laden, initialisieren und zurueckgeben eines fertigen Accounting-Modules.
sub module
{
    my ($self,$h) = @_;

    my $accmod = $h->{accmod};
    $accmod = "Accmod::Default" unless ($accmod);

#    die("acm = $accmod");

#    print "trying to get a '$accmod'\n";


    # Perl-technisches Laden der gewuenschten Accounting-Module-Klasse,
    # "do accmod.pm" etc.
    $self->requireAccmod($accmod);

    # Instanz erzeugen
    my $mod = $accmod->new;

    # my $mod = new $accmod; # FIXME! Warum geht das schief?
    # A: weil Perl unter "new $accmod" ein "new($accmod)" versteht,
    # u. dadurch die Methode "new" im aktuellen Package ausfuehrt. :-X

    return($mod);
}



#
# Usage:
#
# my $a = new acctools::flexacc::accounter({
#                AccmodLibDir => '/usr/local/lib/accounting',
#                });
#
# my @recs = new dbrecs($dbh,"SELECT * FROM accresource
#                              WHERE auftrag = <id>
#                                AND TRUNC(bdate) = '01-JAN-2004'");
#
# my @rgpos = $a->account({
#                rsrcs => \@recs,
#                accmod => 'Accmod::neticProfessional',
#                auftrag => 1235456,
#                });
#
sub account
{
    my ($self,$h) = @_;

    # Holen eines voll initialisierten Accounting-Module
    my $mod = $self->module($h);
    die("module() failed") unless ($mod);


    # Order-Parameters aus AUFTRAG_PAR holen ---------------------
    my %opars = &{$self->{OrderParFunc}}($h->{auftrag});

    foreach my $k (keys %opars)
    {
#	print "setting opar, $k == $opars{$k}\n";
	$mod->opSet($k,$opars{$k});
    }




    
    my $rsrc = {};
    
    foreach my $r (@{$h->{rsrcs}})
    {
	my $rsrcname = $r->{rsrc};
	
#	print "id $r->{id}, rsrc $rsrcname, qty $r->{qty}\n";
	
	# Assertion: Pro Resource-Name darf es nur ein
	# Resource-Entry geben.
	if ($rsrc->{$rsrcname})
	{
	    #$rsrc->{$rsrcname}->{qty} += $r->{qty}; # Workaround
	    die("ASSERT ERROR: rsrc '$rsrcname' redefined");
	}
	else
	{
	    $rsrc->{$rsrcname} = {
		bdate => $r->{bdate},
		edate => $r->{edate},
		qty => $r->{qty}
	    };
	}
	
    } # foreach


    
    # Collect the data to be accounted
    my $accdata = {
#	konto => $konto,
	auftrag => $h->{auftrag},
	rsrc => $rsrc,
    };
    
    
    # If operatorParam records are given, use them to init
    # the accounting module
    # FIXME: Obsolete, should use a callback function
    if ($h->{opRecs})
    {
	die("opRecs-concept is obsolete");
    }
    
    
#    print "mod is $mod, accdata is $accdata\n";
    
    my @rgpos = $mod->account($accdata);
    
    return(@rgpos);
}



sub hook
{
    my ($self,$m,$s) = @_;
    
    my $cpt = new Safe;
    my $result = $cpt->reval($s);
    
    if (!$result)
    {
	print "error while evaluating compartment: $@\n";
	return;
    }
    
    die("config text did not return a hash") unless (ref($result) eq "HASH");
    
    # Write variables to our accounting module.
    $m->varSet($result);
}




sub requireAccmod
{
    my ($self,$accmodClass) = @_;
    
    my $libdir = $self->{AccmodLibDir};
    
    die("no cf entry 'AccmodLibDir' given") unless ($libdir);
    
    my $classFile = $libdir .  "/" . $accmodClass . ".pm";
    
    $classFile =~ s/\:\:/\//g;

    
    die("Cannot find class file $classFile") unless (-f $classFile);


    #require $classFile;

    if (1)
    {
	if (!(my $ret = do $classFile))
	{
#	    print STDERR "Error: $@\n";
	    die("Failed. Error $@");
	}
    }

#    1;
}


1;
