#
# $Id: tset.pm,v 1.4 2004/04/21 15:43:13 rodney Exp $
#
# Funktionen Marke "Super-Simple" fuer netic-style Formfiles, d.h.
# Formfiles mit mehreren Tags gleichen Namens im Header.
#
# my $cf = new acctools::tset($file)
# twrite(\@f,$file)          @array zurueck in eine Form-Datei schreiben
#
# $cf->tdiff                 Aenderungen zwischen zwei Form-Files
#                            finden und human-readable ausgeben
#
# $cf->tset($tag,$value,     Wert eines Tag-Feldes setzen. "Slot"
#      $slot)                ist optional.
#
# tget(\@f,$tag,$slot)       Wert eines Tag-Feldes holen. Slot optional.
#
# tfree(\@f,$tag)            Naechsten freien Slot fuer ein Tag
#                            zurueckliefern. Wenn z.B. zwei Tags
#                            "PPP.Username" existieren, ist der
#                            naechste freie Slot 3.
#
#
package acctools::tset;

use strict;
use IO::Handle;

#
# tread - Form-Datei in ein @array einlesen
#
# $f    - Ref. auf ein @array
# $file - File name
#
# Returns: 0 upon failure, else != 0.
#
# Usage:   my $CF = new tset($file) || die("cannot read $file");
#


sub new {
    my ($this,$file) = @_;
    my $class = ref ($this) || $this; # class or object
    my $self = {};

    bless($self, $class);

    my $fp = new IO::Handle;
    open($fp,"< $file") || return;
    my @f = <$fp>;
    close($fp);


    my @f2 = @f; # Copy

    $self->{f} = \@f;
    $self->{f2} = \@f2;

    return($self);
}


#sub tread {
#    my ($self,$f,$file) = @_;
#
#    my $fp = new IO::Handle;
#
##    print "handle: $fp\n";
#
#    open($fp,"< $file") || return(0);
#    @$f = <$fp>;
#    close($fp);
#
#    return(1);
#} # tread


#
# twrite - Inhalt eines @array in ein File schreiben
#
# $f    - Ref. auf ein @array
# $file - File name
#
# Returns: 0 upon failure, else != 0.
# Usage:   twrite(\@f,$file) || die("cannot write");
#
sub write {
    my ($self,$file) = @_;

    my $f = $self->{f};

    my $fp = new IO::Handle;

    # Don't allow writing to *~-Files
    die("bad file name '$file'") if ($file =~ /~$/);

#    print "handle: $fp\n";
    
    
    # Check, if we have changes
    if (-f $file)
    {
#	my @cmpf;
#	tread(\@cmpf,$file) || return(0);
	my @dif = $self->tdiff;

	if ($#dif == -1)
	{
	    print STDERR "$0: No changes in file '$file'\n";
	    return(1);
	}
    } # endif (-f $file)




    my $bfile = "$file~";

    if (-f $bfile)
    {
	unlink($bfile) || return(0);
    }

    # Get file mode for later re-creation.
    my ($f_dev,$f_ino,$f_mode,$f_nlink,$f_uid,$f_gid,
	$f_rdev,$f_size,$f_atime,$f_mtime,$f_ctime,
	$f_blksize,$f_blocks) = stat($file);


    # Re-create an empty file with good permissions
    if (defined($f_uid))
    {
	return(0) if (!rename($file,$bfile));

	open($fp,"> $file") || return(0);
	close($fp);
	chmod($f_mode,$file); # FIXME, check errors?
	chown($f_uid,$f_gid,$file); # FIXME, check errors?
    }

	
    open($fp,"> $file") || return(0);
    print $fp join('',@$f);
    close($fp);

    return(1);
} # twrite









#
# $f    - Reference auf @f
# $key  - Tag, Form-Key
# $i    - $i'th occurence, start with 0
#
# Usage: $tset->tget($tag,$slot)
#
sub get {
    my ($self,$key,$i) = @_;

#    print STDERR "tget index $i\n";

    return unless ($key);

    my $f = $self->{f};

    my $cnt = 0;
    for (my $n = 0; $n <= $#$f; $n++)
    {
	last if ($$f[$n] =~ /^$/);

	my ($t,$v) = ($$f[$n] =~ /^([^\:]+)\:\s*(.*)$/);

	if ($t eq $key)
	{
	    return($v) if ($cnt++ == $i);
	}
	
#	print "$n $$f[$n]";
    } # end for

#    print "NOMATCH $key\n";

    return;
} # sub get()


sub get_orig
{
    my ($self,$key,$i) = @_;

#    print STDERR "tget index $i\n";

    return unless ($key);

    my $f = $self->{f2};

    my $cnt = 0;
    for (my $n = 0; $n <= $#$f; $n++)
    {
	last if ($$f[$n] =~ /^$/);

	my ($t,$v) = ($$f[$n] =~ /^([^\:]+)\:\s*(.*)$/);

	if ($t eq $key)
	{
	    return($v) if ($cnt++ == $i);
	}
	
#	print "$n $$f[$n]";
    } # end for

#    print "NOMATCH $key\n";

    return;
} # sub get()





#
# $f    - Reference auf @f
# $key  - Tag, Form-Key
# $val  - Zu setzender Wert. Wenn !defined($val), wird die
#         entsprechende Zeile entfernt.
# $i    - $i'th occurence, start with 0
#
#
# Usage: tset(\@f,"Username","mueller")
#

sub set {
    my ($self,$key,$val,$i) = @_;

    my $f = $self->{f};

#    print STDERR "tset index $i\n";

    my $cnt = 0;
    my $n; # Array-Counter
    for ($n = 0; $n <= $#$f; $n++)
    {
	last if ($$f[$n] =~ /^$/);

	my ($t,$v) = ($$f[$n] =~ /^([^\:]+)\:\s*(.*)$/);

	if ($t eq $key)
	{
	    # Key found. Ersetzen.
	    if ($i == $cnt++)
	    {
#		print "line '$$f[$n]', k '$k', v '$v'\n";
		$$f[$n] = "$t: $val\n";
		if (!defined($val))
		{
		    # Zeile entfernen
#		    print "SPLICE!\n";
		    splice(@$f,$n,1);
		}

		return;
	    } # endif
	} # endif key match

#	print "$n $$f[$n]";
    } # end for


#    print "anfuegen!!\n";
    
    splice(@$f,$n,0,"$key: $val\n") if (defined($val));

    return;
} # sub tset()



#
# tfree(\@f,$tag) - Returns the lowest free slot for a tag.
#
# $f    - Ref. auf @array
# $tag  - Tag name
#
# Returns: The lowest free "slot" number for a tag. A slot is
#          considered "free", if the value is empty or undefined.
#
# Usage:   tset(\@f,"Auftrag","1234",tfree(\@f,"Auftrag"));
#
sub free {
    my($self,$tag) = @_;


    my $f = $self->{f};

    my $n;

    for ($n = 0; tget($f,$tag,$n) ne ""; $n++)
    {
    }

    return($n);
} # sub free()




#
# $tset->tdiff Return header differences since last new/write operation
#
# Returns: a @array containing documentation. If nothing changed,
#          an empty array is returned.
#
#
sub diff {
    my ($self) = @_;


    my $f1 = $self->{f};
    my $f2 = $self->{f2};

    my $ret;

    # Fuer Keys, die entweder in @f1 oder @f2 vorkommen,
    # wird pro Key ein Index-Counter gefuehrt.
    my @keys;
    my %kindex;

    # f1
    for (my $n = 0; $n <= $#$f1; $n++)
    {
	last if ($$f1[$n] =~ /^$/);
	my ($k) = ($$f1[$n] =~ /^([^\:]+)\:.*$/);
	next if (!defined($k));
	push(@keys,$k) if (!defined($kindex{$k}));
	$kindex{$k} = 0;
    } # end for

    # f2
    for (my $n = 0; $n <= $#$f2; $n++)
    {
	last if ($$f2[$n] =~ /^$/);
	my ($k) = ($$f2[$n] =~ /^([^\:]+)\:.*$/);
	next if (!defined($k));
	push(@keys,$k) if (!defined($kindex{$k}));
	$kindex{$k} = 0;
    } # end for


    my @ret;

    my $changes;
    my $ccnt = 0;
    do
    {
	$changes = 0;

	# Pro Key: Files durchgehen
	foreach my $k (@keys)
	{
#	    print "k $k\n";

	    my $v1 = $self->get_orig($k,$kindex{$k});
	    my $v2 = $self->get($k,$kindex{$k});
#	    print "v1 $v1, v2 $v2\n";

	    
	    next if ((defined($v1) && defined($v2) && $v1 eq $v2) ||
		     !defined($v1) && !defined($v2));

#	    print "found changes for key $k\[$kindex{$k}\]\n";



	    $changes++; # Prepare for next round
	    $ccnt++;

	    $v1 = "'$v1'" if (defined($v1));
	    $v2 = "'$v2'" if (defined($v2));
	    
	    $v1 = "NULL" if (!defined($v1));
	    $v2 = "NULL" if (!defined($v2));

	    my $l = "$k";
	    $l .= "\[$kindex{$k}\]" if ($kindex{$k} > 0);
	    $l .= ": $v1 -> $v2";
	    push(@ret,"- $l\n");

	    $kindex{$k}++;
	} # foreach $k
    } while ($changes);



    return unless ($ccnt > 0);

    my $e = "- $ccnt changed ";
    if ($ccnt == 1)
    {
	$e .= "entry";
    }
    else
    {
	$e .= "entries";
    }

    # File size changed?
    if ($#$f1 != $#$f2)
    {
	$e .= "; changed from " . ($#$f1 + 1) . " to " .
	    ($#$f2 + 1) . " lines";
    }

    push(@ret,"$e.\n");

    # Add a line with the users login name
    my $login = $ENV{"LOGNAME"};
    $login = $ENV{"USER"} if (!defined($login));
    $login = "UID " . getuid() if (!defined($login));
    
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
	localtime(time());
    
    $year += 1900;
    $mon += 1;
    

#    my $ti = sprintf("%04d-%02d-%02d %02d:%02d:%02d -$login\n",
#		     $year,$mon,$mday,$hour,$min,$sec);

    my $ti = localtime(); # FIXME, doesn't display the timezone!
    chomp($ti);
    $ti .= " -$login\n";

    unshift(@ret,$ti);

#    print join('',@ret);

    unshift(@ret,"\n") if ($#ret > -1);

    return(@ret);

} # tdiff(\@f1, \@f2)


sub append
{
    my ($self,@args) = @_;

    my $f = $self->{f};

    push(@$f,@args);
}

1;

